@@ -24,11 +24,11 @@
 #include <ctime>
 #include <clockd/libtime.h>
 #include <dbus/dbus-glib-lowlevel.h>
-extern "C"{
+
 
 #include <hildon/hildon.h>
 #include <libhildondesktop/libhildondesktop.h>
-}
+
 #include <CMulticalendar.h>
 #include <CalInterface.h>
 #include <CComponent.h>
@@ -39,8 +39,9 @@ extern "C"{
 #include "cal-home-calendar-util.h"	
 #include "cal-home-settings-dlg.h"	
 
+
 extern "C" {
-HD_DEFINE_PLUGIN_MODULE(CalHomePlugin, cal_home_plugin, HD_TYPE_HOME_PLUGIN_ITEM)
+  HD_DEFINE_PLUGIN_MODULE(CalHomePlugin, cal_home_plugin, HD_TYPE_HOME_PLUGIN_ITEM)
 }
 
 
@@ -165,9 +166,10 @@ cal_update_time(gpointer data);
 static void
 register_time_out_handler(CalHomePlugin* desktop_plugin)
 {
+  static int seconds_in_day = 24*60*60;
   struct tm now;
   time_get_local(&now);
-  desktop_plugin->time_out_handler = g_timeout_add_seconds((23-now.tm_hour)*3600 - 50 * now.tm_min - now.tm_sec, 
+  desktop_plugin->time_out_handler = g_timeout_add_seconds(seconds_in_day-(now.tm_hour*3600 + 60 * now.tm_min + now.tm_sec),
 							   cal_update_time, 
 							   (gpointer) desktop_plugin);
 }
@@ -178,8 +180,12 @@ cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin);
 static gboolean
 cal_update_time(gpointer data)
 {
-  cal_update_if_visible_and_date_changed((CalHomePlugin*)data);
-  register_time_out_handler((CalHomePlugin*)data);
+  if(CAL_HOME_IS_HOME_PLUGIN(data))
+  {
+    CAL_HOME_PLUGIN(data)->update_view =  TRUE;
+    cal_update_if_visible_and_date_changed(CAL_HOME_PLUGIN(data));
+    register_time_out_handler((CalHomePlugin*)data);
+  }
   return FALSE;
 }
 
@@ -189,13 +195,14 @@ handle_time_format_changed(GConfClient* client,
 			   GConfEntry* entry,
 			   gpointer user_data)
 {
+  g_return_if_fail(CAL_HOME_IS_HOME_PLUGIN(user_data));
   CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(user_data);
   desktop_plugin->has_24_time_format = cal_24_time_format();
   desktop_plugin->update_view = TRUE;
 }
 
-static void
-cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin)
+static gboolean 
+today_date_changed(CalHomePlugin* desktop_plugin)
 {
   struct tm now;
   time_get_local(&now);
@@ -204,17 +211,26 @@ cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin)
      now.tm_mon != desktop_plugin->date_today.tm_mon ||
      now.tm_year != desktop_plugin->date_today.tm_year);
 
-  if(!time_differ)
+  return time_differ;
+}
+
+static void
+cal_update_if_visible_and_date_changed(CalHomePlugin* desktop_plugin)
+{
+  if(!today_date_changed(desktop_plugin))
   {
     return;
   }
   gboolean on;
   g_object_get(GTK_WIDGET(desktop_plugin), "is-on-current-desktop", &on, NULL);
 
+  
   if(on)
   {
     render_day_on_title(desktop_plugin);
     cal_home_load_db_events(desktop_plugin);
+    cal_widget_resize_for_cal_size(desktop_plugin);
+
     //    gtk_widget_queue_draw(GTK_WIDGET(desktop_plugin));    
   }
   else
@@ -234,10 +250,11 @@ handle_db_change(DBusConnection *bus,
 		 DBusMessage *msg, 
 		 gpointer data)
 {
-  if(dbus_message_is_signal(msg,CALENDAR_SERVICE, CALENDAR_DBUS_LISTENER_SIGNAL))
-  {  
-    CAL_HOME_PLUGIN(data)->update_view = TRUE;
-  }
+  if(CAL_HOME_IS_HOME_PLUGIN(data))
+    if(dbus_message_is_signal(msg,CALENDAR_SERVICE, CALENDAR_DBUS_LISTENER_SIGNAL))
+    {  
+      CAL_HOME_PLUGIN(data)->update_view = TRUE;
+    }
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
@@ -245,6 +262,8 @@ static void
 cal_on_current_desktop(GtkWidget* widget,
 		       gpointer data)
 {
+  g_return_if_fail(CAL_HOME_IS_HOME_PLUGIN(widget));
+
   CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(widget);
   gboolean on;
   g_object_get(widget, "is-on-current-desktop", &on, NULL);
@@ -254,10 +273,12 @@ cal_on_current_desktop(GtkWidget* widget,
     {
       register_time_out_handler(desktop_plugin);
     }
-    if(desktop_plugin->update_view)
+    if(today_date_changed(desktop_plugin) ||
+       desktop_plugin->update_view)
     {
       render_day_on_title(desktop_plugin);
       cal_home_load_db_events(desktop_plugin);
+      cal_widget_resize_for_cal_size(desktop_plugin);
     }
   }
   else if(desktop_plugin->time_out_handler)
@@ -403,7 +424,6 @@ cal_realize(GtkWidget* widget)
   cal_home_load_db_events(desktop_plugin);
   cal_widget_resize_for_cal_size(desktop_plugin);
   GTK_WIDGET_CLASS(cal_home_plugin_parent_class)->realize(widget);
-
 }
 
 static void
@@ -486,10 +506,10 @@ cal_render_background_auto(cairo_t* cr, CalHomePlugin* desktop_plugin)
   }
 }
 
-static void 
-cal_expose(GtkWidget* widget, GdkEventExpose *event, gpointer data)
+static gboolean
+cal_expose(GtkWidget* widget, GdkEventExpose *event)
 {
-  CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(data);
+  CalHomePlugin* desktop_plugin = CAL_HOME_PLUGIN(widget);
   cairo_t* cr;
   cr = gdk_cairo_create(GDK_DRAWABLE(widget->window));
   gdk_cairo_set_source_pixbuf(cr, desktop_plugin->applet_bg, 0, 0);
@@ -512,6 +532,7 @@ cal_expose(GtkWidget* widget, GdkEventExpose *event, gpointer data)
     cal_render_background_auto(cr, desktop_plugin);
   }
   cairo_destroy(cr);
+  return GTK_WIDGET_CLASS(cal_home_plugin_parent_class)->expose_event(widget, event);
 }
 
 static void
@@ -564,25 +585,25 @@ register_dbus_signal_on_time_changed(CalHomePlugin *desktop_plugin)
 static void
 register_dbus_signal_on_db_changed(CalHomePlugin *desktop_plugin)
 {
-  desktop_plugin->dbus_session_conn = hd_home_plugin_item_get_dbus_g_connection(&desktop_plugin->hitem, 
-										DBUS_BUS_SESSION, 
-										NULL );
-  if(desktop_plugin->dbus_session_conn)
-  {
-    desktop_plugin->dbus_calendar_proxy = dbus_g_proxy_new_for_name(desktop_plugin->dbus_session_conn,
-								    CALENDAR_SERVICE,
-								    CALENDAR_PATH,
-								    CALENDAR_INTERFACE);
-
-  }
-  dbus_bus_add_match(dbus_g_connection_get_connection(desktop_plugin->dbus_session_conn), 
-		     "type='signal',interface='com.nokia.calendar'", 
-		     NULL);
-
-  dbus_connection_add_filter(dbus_g_connection_get_connection(desktop_plugin->dbus_session_conn), 
-			     handle_db_change, 
-			     desktop_plugin, 
-			     NULL);
+  desktop_plugin->dbus_session_conn = hd_home_plugin_item_get_dbus_g_connection(&desktop_plugin->hitem,  
+ 										DBUS_BUS_SESSION,  
+ 										NULL ); 
+  if(desktop_plugin->dbus_session_conn) 
+  { 
+    desktop_plugin->dbus_calendar_proxy = dbus_g_proxy_new_for_name(desktop_plugin->dbus_session_conn, 
+								    CALENDAR_SERVICE, 
+								    CALENDAR_PATH, 
+								    CALENDAR_INTERFACE); 
+    
+  } 
+  dbus_bus_add_match(dbus_g_connection_get_connection(desktop_plugin->dbus_session_conn),   
+  		     "type='signal',interface='com.nokia.calendar'",   
+  		     NULL);  
+  
+  dbus_connection_add_filter(dbus_g_connection_get_connection(desktop_plugin->dbus_session_conn),   
+  			     handle_db_change,   
+  			     desktop_plugin,   
+  			     NULL);  
 }
 
 static void
@@ -638,10 +659,9 @@ cal_home_plugin_init(CalHomePlugin *desktop_plugin)
   g_signal_connect(desktop_plugin, "show-settings", G_CALLBACK(cal_show_settings_dialog), NULL);
   g_signal_connect(desktop_plugin, "notify::is-on-current-desktop", G_CALLBACK(cal_on_current_desktop), NULL);
   g_signal_connect(GTK_CONTAINER(contents), "button-release-event", G_CALLBACK(cal_home_button_release), desktop_plugin);
-
-  g_signal_connect(G_OBJECT(contents), "expose_event", G_CALLBACK(cal_expose), desktop_plugin);
+  //  g_signal_connect(G_OBJECT(contents), "expose_event", G_CALLBACK(cal_expose), desktop_plugin);
   register_time_out_handler(desktop_plugin);
-  register_gconf_notify_on_timeformat_changed(desktop_plugin);  
+  register_gconf_notify_on_timeformat_changed(desktop_plugin);
   register_dbus_signal_on_time_changed(desktop_plugin);
   register_dbus_signal_on_db_changed(desktop_plugin);
 }
@@ -656,15 +676,29 @@ cal_plugin_finalize(GObject *object)
     desktop_plugin->time_out_handler = 0;
   }
 
+  if(desktop_plugin->gconf_notify_handler)
+  {
+    GConfClient* client = gconf_client_get_default();
+    if(GCONF_IS_CLIENT(client))
+    {
+      gconf_client_notify_remove(client,
+				 desktop_plugin->gconf_notify_handler);
+      desktop_plugin->gconf_notify_handler = 0;
+    }
+  }
   if(desktop_plugin->dbus_clockd_proxy)
     g_object_unref(desktop_plugin->dbus_clockd_proxy);
 
   if(desktop_plugin->dbus_calendar_proxy)
     g_object_unref(desktop_plugin->dbus_calendar_proxy);
 
-  dbus_g_connection_unref(desktop_plugin->dbus_system_conn);  
-  dbus_g_connection_unref(desktop_plugin->dbus_session_conn);  
+  dbus_connection_remove_filter(dbus_g_connection_get_connection(desktop_plugin->dbus_session_conn),   
+				handle_db_change,   
+				object);
+  dbus_g_connection_unref(desktop_plugin->dbus_system_conn);   
+  dbus_g_connection_unref(desktop_plugin->dbus_session_conn);
   g_free(desktop_plugin->iD);
+  g_object_unref(desktop_plugin->applet_bg);
   G_OBJECT_CLASS (cal_home_plugin_parent_class)->finalize (object);
 }
 
@@ -673,6 +707,7 @@ cal_home_plugin_class_init(CalHomePluginClass *klass)
 {
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
   widget_class->realize = cal_realize;
+  widget_class->expose_event = cal_expose;
   G_OBJECT_CLASS(klass)->finalize = cal_plugin_finalize;
 }
 
